@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package App::Uni;
 # ABSTRACT: command-line utility to find or display Unicode characters
-$App::Uni::VERSION = '9.001';
+$App::Uni::VERSION = '9.002';
 #pod =encoding utf8
 #pod
 #pod =head1 NAME
@@ -47,46 +47,72 @@ use 5.10.0; # for \v
 use warnings;
 
 use charnames ();
+use Encode qw(encode_utf8);
+use Getopt::Long;
+use List::Util qw(max);
 use Unicode::GCString;
 
+sub _do_help {
+  my $class = shift;
+
+  die
+    join qq{\n  }, join(qq{\n}, @_, @_ ? "" : (), "usage:"),
+    "uni SEARCH-TERMS...    - find codepoints with matching names or values",
+    "uni [-s] ONE-CHARACTER - print the codepoint and name of one character",
+    "uni -n SEARCH-TERMS... - find codepoints with matching names",
+    "uni -c STRINGS...      - print out the codepoints in a string",
+    "uni -u CODEPOINTS...   - look up and print hex codepoints",
+    "",
+    "Other switches:",
+    "    -8                 - also show the UTF-8 bytes to encode\n";
+}
+
 sub run {
   my ($class, @argv) = @_;
 
-  if (! @argv or $argv[0] eq '--help') {
-    die join qq{\n  }, "usage:",
-      "uni SEARCH-TERMS...    - find codepoints with matching names or values",
-      "uni [-s] ONE-CHARACTER - print the codepoint and name of one character",
-      "uni -n SEARCH-TERMS... - find codepoints with matching names",
-      "uni -c STRINGS...      - print out the codepoints in a string",
-      "uni -u CODEPOINTS...   - look up and print hex codepoints\n";
+  my %opt;
+  {
+    my $exit;
+    local @ARGV = @argv;
+    GetOptions(
+      "c" => \$opt{explode},
+      "u" => \$opt{u_numbers},
+      "n" => \$opt{names},
+      "s" => \$opt{single},
+      "8" => \$opt{utf8},
+      "help|?" => \$opt{help},
+    );
+    @argv = @ARGV;
   }
 
-  my $todo;
-  $todo = \&do_explode    if @argv && $argv[0] eq '-c';
-  $todo = \&do_u_numbers  if @argv && $argv[0] eq '-u';
-  $todo = \&do_names      if @argv && $argv[0] eq '-n';
-  $todo = \&do_single     if @argv && $argv[0] eq '-s';
+  $class->_do_help if $opt{help};
 
-  shift @argv if $todo;
+  my $n = grep { $_ } @opt{qw(explode u_numbers names single)};
 
-  if (grep /\A-./, @argv) {
-    die "uni: only one swich allowed!\n" if $todo;
-    die "uni: unknown switch $argv[0]\n";
-  }
+  $class->_do_help("ERROR: only one mode switch allowed!") if $n > 1;
 
-  $todo //= @argv == 1 && length $argv[0] == 1
-          ? \&do_single
-          : \&do_dwim;
+  $class->_do_help if ! @argv;
 
-  $todo->(\@argv);
+  my $todo  = $opt{explode}                       ? \&do_explode
+            : $opt{u_numbers}                     ? \&do_u_numbers
+            : $opt{names}                         ? \&do_names
+            : $opt{single}                        ? \&do_single
+            : @argv == 1 && length $argv[0] == 1  ? \&do_single
+            :                                       \&do_dwim;
+
+  $todo->(\@argv, \%opt);
 }
 
 sub do_single {
-  print_chars(@_);
+  my @chars    = grep { length } @{ $_[0] };
+  if (my @too_long = grep { length > 1 } @chars) {
+    die "some arguments were too long for use with -s: @too_long\n";
+  }
+  print_chars(\@chars, $_[1]);
 }
 
 sub do_explode {
-  print_chars( explode_strings(@_) );
+  print_chars( explode_strings($_[0]), $_[1] );
 }
 
 sub explode_strings {
@@ -103,13 +129,30 @@ sub explode_strings {
 }
 
 sub do_u_numbers {
-  print_chars( chars_by_u_numbers(@_) );
+  print_chars( chars_by_u_numbers($_[0]), $_[1] );
 }
 
 sub print_chars {
-  my ($chars) = @_;
+  my ($chars, $opt) = @_;
+
+  my @to_print = $opt->{utf8}
+               ? (map {; [ $_ => defined && encode_utf8($_) ] } @$chars)
+               : (map {; [ $_ ] } @$chars);
+
+  my $width;
+  if ($opt->{utf8}) {
+    my $max_bytes = 0;
+    for my $todo (@to_print) {
+      $max_bytes = max($max_bytes, length $todo->[1]);
+      last if $max_bytes == 4; # maximum ever
+    }
+
+    $width = 2 * $max_bytes + $max_bytes - 1;
+  }
+
+  for my $todo (@to_print) {
+    my ($c, $u) = @$todo;
 
-  for my $c (@$chars) {
     unless (defined $c) { print "\n"; next }
 
     my $c2 = Unicode::GCString->new($c);
@@ -130,7 +173,12 @@ sub print_chars {
 
     my $chr  = ord($c);
     my $name = charnames::viacode($chr);
-    printf "%s- U+%05X - %s\n", $p, $chr, $name;
+    my $utf8 = $opt->{utf8}
+             ? (sprintf " - %${width}s",
+                 join q{ }, map {; sprintf '%02X', ord } split //, $u)
+             : '';
+
+    printf "%s- U+%05X%s - %s\n", $p, $chr, $utf8, $name;
   }
 }
 
@@ -141,9 +189,9 @@ sub chars_by_u_numbers {
 }
 
 sub do_names {
-  my ($terms) = @_;
+  my ($terms, $opt) = @_;
 
-  print_chars( chars_by_name( $terms ) );
+  print_chars( chars_by_name( $terms ), $opt );
 }
 
 sub chars_by_name {
@@ -191,9 +239,9 @@ sub smerge {
 }
 
 sub do_dwim {
-  my ($argv) = @_;
+  my ($argv, $opt) = @_;
   my $chars = chars_by_name($argv, { match_codepoints => 1 });
-  print_chars($chars);
+  print_chars($chars, $opt);
 }
 
 1;
@@ -210,7 +258,7 @@ App::Uni - command-line utility to find or display Unicode characters
 
 =head1 VERSION
 
-version 9.001
+version 9.002
 
 =head1 SYNOPSIS
 
